/*
    Isometric Computation Tools -- extension for Stencyl 3.3+
    Robert L. Watson
    2015

    http://www.anorthogonaluniverse.com/public-code/isometric.php
*/

/*
    File:       Isometric.hx
    Purpose:    Provide mathematical tools for isometric/scene coordinate calculations
*/

import com.stencyl.Engine;


class Isometric
{
    /*
        Provide some default values in case none are provided.
        The default values are set to project from an origin mid-way across a scene of 640px width.
    */

	static var startX = 320;
	static var startY = 112;

    static var tileSize = 32;

	static var levelHeight = 16;


    /*
        Allow to set parameters
    */

	public static function setStartX( val:Int )
	{
		startX = val;
	}

	public static function setStartY( val:Int )
	{
		startY = val;
	}

	public static function setLevelHeight( val:Int )
	{
		levelHeight = val;
	}

	public static function setTileSize( val:Int )
	{
		tileSize = val;
	}


    /*
        Read parameters
    */

	public static function getStartX( ):Int
	{
		return startX;
	}

	public static function getStartY( ):Int
	{
		return startY;
	}

	public static function getLevelHeight( ):Int
	{
		return levelHeight;
	}

	public static function getTileSize( ):Int
	{
		return tileSize;
	}


    /*
        Convert from grid row,column to scene coordinates
    */

	public static function xSceneAtRC( row:Int, col:Int ):Int
	{
        return xSceneAtGridXY( col * tileSize, row * tileSize );
	}

	public static function ySceneAtRC( row:Int, col:Int, level:Int ):Int
	{
        return ySceneAtGridXYZ( col * tileSize, row * tileSize, level * levelHeight );
	}


    /*
        Convert from grid coordinates to scene coordinates
    */

	public static function xSceneAtGridXY( gx:Int, gy:Int ):Int
	{
		return Math.floor( gx - gy + startX );
	}

	public static function ySceneAtGridXY( gx:Int, gy:Int, level:Int ):Int
	{
		return Math.floor( ( gx / 2 ) + ( gy / 2 ) + startY + ( -1 * levelHeight * level ) );
	}

	public static function ySceneAtGridXYZ( gx:Int, gy:Int, gz:Int ):Int
	{
		return Math.floor( ( gx / 2 ) + ( gy / 2 ) + startY + ( -1 * gz) );
	}


    /*
        Convert from scene coordinates to grid row,column
    */

	public static function rowAtSceneXY( sx:Int, sy:Int, level:Int ):Int
	{
        return Math.floor( yGridAtSceneXY( sx, sy, levelHeight * level ) / tileSize );
	}

	public static function columnAtSceneXY( sx:Int, sy:Int ):Int
	{
        return Math.floor( xGridAtSceneXY( sx, sy ) / tileSize );
	}


    /*
        Convert from grid coordinates to grid row,column
    */

	public static function rowAtGridY( y:Int ):Int
	{
		return Math.floor( y / tileSize );
	}

	public static function columnAtGridX( x:Int ):Int
	{
		return Math.floor( x / tileSize );
	}

	public static function levelAtGridZ( z:Int ):Int
	{
		return Math.floor( z / levelHeight );
	}


    /*
        Convert from scene coordinates to grid coordinates
    */

	public static function xGridAtSceneXY( sx:Int, sy:Int ):Int
	{
        return Math.floor( ( sx / 2 ) + sy + ( startX / -2 ) - startY );
	}

	public static function yGridAtSceneXY( sx:Int, sy:Int, level:Int ):Int
	{
        return yGridAtSceneXYZ( sx, sy, levelHeight * level );
	}

	public static function yGridAtSceneXYZ( sx:Int, sy:Int, gz:Int ):Int
	{
        return Math.floor( ( sx / -2 ) + sy + gz + ( startX / 2 ) - startY );
	}
}

